<?php
/**
 * Image download authorisation script
 *
 * To use, in LocalSettings.php set $wgUploadDirectory to point to a non-public
 * directory, and $wgUploadPath to point to this file. Also set $wgWhitelistRead
 * to an array of pages you want everyone to be able to access. Your server must
 * support PATH_INFO, CGI-based configurations generally don't.
 */
define( 'MW_NO_OUTPUT_COMPRESSION', 1 );
require_once( './includes/WebStart.php' );
wfProfileIn( 'img_authNS.php' );
require_once( './includes/StreamFile.php' );

if( !isset( $_SERVER['PATH_INFO'] ) ) {
	wfDebugLog( 'img_auth', "missing PATH_INFO" );
	wfForbidden();
}

# Get filenames/directories
wfDebugLog( 'img_authNS', "PATH_INFO is: " . $_SERVER['PATH_INFO'] );
$filename = realpath( $wgUploadDirectory . $_SERVER['PATH_INFO'] );
$realUploadDirectory = realpath( $wgUploadDirectory );

# $imageName = $wgContLang->getNsText( NS_IMAGE ) . ":" . wfBaseName( $_SERVER['PATH_INFO'] );

## Added for Namespace detection
$nsbase = "";
$subdirs = explode('/',$_SERVER['PATH_INFO']);

if (strlen($subdirs[1]) == 3 && is_numeric($subdirs[1]) )  {
## this sucks but wgContLang->getNsText( $subdirs[1] ) doesn't work!
	$nst = $wgCanonicalNamespaceNames[$subdirs[1]];
##
	if ($nst) {
		$nsbase = $nst.":";	
		wfDebugLog( 'img_authNS', "Found ns: $nsbase." );
	}
}else {
	$nst = false;
}
$imageName = $wgContLang->getNsText( NS_IMAGE ) . ":" . $nsbase . wfBaseName( $_SERVER['PATH_INFO'] );
##

# Check if the filename is in the correct directory
if ( substr( $filename, 0, strlen( $realUploadDirectory ) ) != $realUploadDirectory ) {
	wfDebugLog( 'img_authNS', "requested path not in upload dir: $filename" );
	wfForbidden();
}

if ( is_array( $wgWhitelistRead ) && !in_array( $imageName, $wgWhitelistRead ) && !$wgUser->getID() ) {
	wfDebugLog( 'img_authNS', "not logged in and requested file not in whitelist: $imageName" );
	wfForbidden();
}

if( !file_exists( $filename ) ) {
	wfDebugLog( 'img_authNS', "requested file does not exist: $filename" );
	wfForbidden();
}
if( is_dir( $filename ) ) {
	wfDebugLog( 'img_authNS', "requested file is a directory: $filename" );
	wfForbidden();
}
## Check to see if NS is locked for read and if so, does the user have rights
wfDebugLog( 'img_authNS', "nst: $nst" );
if ($nst){
	$groups = @$wgNamespacePermissionLockdown[$subdirs[1]]['read'];
	wfDebugLog( 'img_authNS', "nst: $nst" );
	wfDebugLog( 'img_authNS', "Matched: ".print_r($groups,True) );
	if (!$groups) $groups = @$wgNamespacePermissionLockdown['*']['read'];
	if (!$groups) $groups = @$wgNamespacePermissionLockdown[$subdirs[1]]['*'];
	if ($groups){
		wfDebugLog( 'img_authNS', "Matched: ".print_r($groups,True) );
		$ugroups = $wgUser->getEffectiveGroups();
		$match = array_intersect($ugroups, $groups);
		if (!$match) {
			wfForbidden();
		}
	}
}
##

# Write file
wfDebugLog( 'img_auth', "streaming file: $filename" );
wfStreamFile( $filename );
wfLogProfilingData();

function wfForbidden() {
	header( 'HTTP/1.0 403 Forbidden' );
	header( 'Content-Type: text/html; charset=utf-8' );
	print
"<html><body>
<h1>Access denied</h1>
<p>You need to log in to access files on this server</p>
</body></html>";
	wfLogProfilingData();
	exit;
}

?>
